require(genlasso)
require(Matrix)

source("pglasso.R") #road function
source("grid.cluster.R")
source("grid penalty and poisson function.txt")
source("sensitivity and specificity calculation.R")
#==============================================================================================================
#==============================================================================================================
# Example 1-1: Null cluster without a covariate

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

#E(Y)
beta0 <- 2    
mu_i <- Ei * exp(beta0)

#penalty matrix : D
n<-10   #raw data is 10*10 matrix
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <- seq(7,15,1)   
list.gamma  <- seq(0.5,2.5,0.5)  
list.delta <- 0
length(list.lambda)*length(list.gamma)*length(list.delta)

id <-  70 

raw <- list()  #raw data for simulation
new.raw <- list() #raw data for MSPE
output <- list()

run.number <- 1      
system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=NULL, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta, gconv.eps=1e-3, conv.eps=1e-3)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number +1
  }
)

#specificity
res.check <- classic.no.cluster(output=output, First.DataNum=1, Last.DataNum=500)
mean(res.check$spec.val);
sd(res.check$spec.val)

#==============================================================================================================
#==============================================================================================================
#Example 1-2: Null cluster with a covariate

#independent variable
set.seed(12345)
X <- matrix(runif(n=100,min=-0.5,max=0.5),100,1) #matrix form!! 

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

#E(Y)
beta0 <- 2  
beta1 <- 0.5  
mu_i <- Ei * exp(beta0+beta1*X)

#penalty matrix : D
n<-10             #raw data is 10*10 matrix
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <- seq(10,30,5)
list.gamma <- seq(0.7,1,0.1) 
list.delta <- seq(0.7,1,0.1)

id <-70  

raw <- list()  #raw data for simulation
new.raw <- list() # raw data for MSPE
output <- list()

run.number <- 1       #fittingϴ data number

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=X, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <- run.number +1
  }
)

#specificity
res.check <- classic.no.cluster(output=output, First.DataNum=1, Last.DataNum=500)
mean(res.check$spec.val);
sd(res.check$spec.val)

#==============================================================================================================
#==============================================================================================================
#Example 2-1: Square cluster without a covariate

#cluster location
n<-10         #raw data is 10*10 matrix
size <- 3                               #inteded cluster size
start.point <- 34                      	#cluster start  cell
standard <- seq(start.point,(start.point+size-1),1)  
intend <- standard         #intended cluster location

for( k in 1:(size-1)){
  intend <- c(intend, standard + (n*k))
} 		    

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

# E(Y)
beta0 <- 2   
alpha <- 1  
mu_i <- Ei * exp(beta0)
mu_i[intend] <- Ei[intend] * exp(beta0+alpha) 

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <- seq(40,70,5) 
list.gamma <- seq(0.1,0.8,0.1) 
list.delta <- 0

id <-70 

raw <- list()  #raw data for simulation
new.raw <- list()  #raw data for MSPE
output <- list()

run.number <- 1      

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  #m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=NULL, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number+1
  }
)


#sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

#specificity
mean(res.check$spec.val)
sd(res.check$spec.val)

#sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)

#==============================================================================================================
#==============================================================================================================
#Example 2-2: Square cluster with a covariate

#cluster location
n<-10         #raw data is 10*10 matrix
size <- 3                               #inteded cluster size
start.point <- 34                      	#cluster start  cell
standard <- seq(start.point,(start.point+size-1),1)  
intend <- standard         #intended cluster location
for( k in 1:(size-1)){
  intend <- c(intend, standard + (n*k))
} 		    

#independent variable
set.seed(12345)
X <- matrix(runif(n=100,min=-0.5,max=0.5),100,1) #matrix form!! 

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

#E(Y)
beta0 <- 2  
beta1 <- 0.5  
alpha <- 1  #alpha is ouput$mu's ture value
mu_i <- Ei * exp(beta0+beta1*X)      #backgroud
mu_i[intend] <- Ei[intend] * exp(beta0+alpha+beta1*X[intend])    #cluster

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <- seq(29,35,2)   
list.gamma <-  seq(0.1,0.5,0.1) 
list.delta <- c(0.1,0.3,0.5)  

id <-70  

raw <- list()  #raw data for simulation
new.raw <- list() # raw data for MSPE
output <- list()

run.number <- 1        

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=X, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number +1
  }
)


#sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

#specificity
mean(res.check$spec.val)
sd(res.check$spec.val)

#sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)

#==============================================================================================================
#==============================================================================================================
#Example 3-1: Triangle-shape cluster without a covariate

#cluster location
n <- 10         #raw data is 10*10 matrix
intend <- c(34,44,54,64,45,55,65,56,66,67)

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

# E(Y)
beta0 <- 2   
alpha <- 1  
mu_i <- Ei * exp(beta0)
mu_i[intend] <- Ei[intend] * exp(beta0+alpha) 

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting

#tuning parameter
list.lambda <- seq(9,21,2)
list.gamma <- seq(0.5,1.8,0.1)
list.delta <- 0


id <-70 

raw <- list()  #raw data for simulation
new.raw <- list()  #raw data for MSPE
output <- list()

run.number <- 1       

system.time(
  
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=NULL, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number +1
  }
)

###sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

###specificity
mean(res.check$spec.val)
sd(res.check$spec.val)
#res.check$spec.val

###sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)

#==============================================================================================================
#==============================================================================================================
#Example 3-2: Triangle-shape cluster with a covariate

#cluster location
n<-10         #raw data is 10*10 matrix
intend <- c(34,44,54,64,45,55,65,56,66,67)

#independent variable
set.seed(12345)
X <- matrix(runif(n=100,min=-0.5,max=0.5),100,1) #matrix form!! 

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

#E(Y)
beta0 <- 2  
beta1 <- 0.5  
alpha <- 1  #alpha is ouput$mu's ture value
mu_i <- Ei * exp(beta0+beta1*X)      #backgroud
mu_i[intend] <- Ei[intend] * exp(beta0+alpha+beta1*X[intend])    #cluster

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <-  seq(20,50,5)
list.gamma <- seq(0.1,0.7,0.2)
list.delta <- seq(0,0.3,0.1)

id <-70  

raw <- list()  #raw data for simulation
new.raw <- list() # raw data for MSPE
output <- list()

run.number <- 1        

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=X, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number +1
  }
)

###sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

###specificity
mean(res.check$spec.val)
sd(res.check$spec.val)
#res.check$spec.val

###sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)

#==============================================================================================================
#==============================================================================================================
#Example 4-1: Two square clusters without a covariate

#cluster location
n<-10         #raw data is 10*10 matrix
intend <- c(27,28,37,38,63,64,73,74)

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

# E(Y)
beta0 <- 2   
alpha <- 1  
mu_i <- Ei * exp(beta0)
mu_i[intend] <- Ei[intend] * exp(beta0+alpha) 

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <- seq(10,25,5) 
list.gamma <- seq(0.2,1.2,0.2)  
list.delta <- 0

id <-70 

raw <- list()  #raw data for simulation
new.raw <- list()  #raw data for MSPE
output <- list()

run.number <- 1       

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=NULL, D=D, offset=Ei, list.lambda=list.lambda,
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <-run.number +1
  }
)
###sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

###specificity
mean(res.check$spec.val)
sd(res.check$spec.val)
#res.check$spec.val

###sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)

#==============================================================================================================
#==============================================================================================================
#Example 4-2: Two square clusters with a covariate

#cluster location
n<-10         #raw data is 10*10 matrix
intend <- c(27,28,37,38,63,64,73,74)

#independent variable
set.seed(12345)
X <- matrix(runif(n=100,min=-0.5,max=0.5),100,1) #matrix form!! 

#offset
set.seed(12345)
Ei <- runif(n=100,min=2,max=86)

#E(Y)
beta0 <- 2  
beta1 <- 0.5  
alpha <- 1  #alpha is ouput$mu's ture value
mu_i <- Ei * exp(beta0+beta1*X)      #backgroud
mu_i[intend] <- Ei[intend] * exp(beta0+alpha+beta1*X[intend])    #cluster

#penalty matrix : D
D <- D.matrix(n)  #fusion penalty => matrix.res

#simulation data generating and model fitting
#tuning parameter
list.lambda <-  c(0.3,0.5,0.7,0.9)  
list.gamma <- seq(50,150,30) 
list.delta <- c(0, 0.1,0.3) 

id <-70  

raw <- list()  #raw data for simulation
new.raw <- list() # raw data for MSPE
output <- list()

run.number <- 1        

system.time(
  for(m in 1:500){  
    set.seed(1000*id+m)  # ID is computer number, m is number of repeatation
    raw[[m]] <-  sapply(mu_i,Poisson)   #m-th raw data
    y <- raw[[m]]
    print(run.number)
    tmp.fit <- grid.clusters(do.init.0=TRUE, y=y, X=X, D=D, offset=Ei, list.lambda=list.lambda, 
                             list.gamma=list.gamma, list.delta=list.delta)
    if(tmp.fit$feasibility==TRUE)
      output[[m]] <- tmp.fit
    else{   
      output[[m]] <- NA
    }
    set.seed(1000*id+30*m)
    new.raw[[m]] <- sapply(mu_i,Poisson)   #m-th new raw data
    run.number <- run.number +1
  }
)

###sensitivity and specificity calculation
res.check <- classic.cluster(output=output, intend=intend, First.DataNum=1, Last.DataNum=500)

###specificity
mean(res.check$spec.val)
sd(res.check$spec.val)
#res.check$spec.val

###sensitivity
mean(res.check$sens.val)
sd(res.check$sens.val)